@@ -13,6 +13,4 @@
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #    GNU General Public License for more details.
 
-"""Module containing functions to work with the Buhmann FAFB synapses."""
-
-from .synapses import *
+"""Module containing low-level functions to work with the Buhmann FAFB synapses."""
